
/*
    FROM: National Olympiad 2008, K5
    PROB: lesna
    KEYW: tree, dynamic programming, lowest-common-ancestor in constant time
*/

// Hopefully the input is really a tree.

#include <cstdio>
#include <vector>
#include <cstring>
#include <iostream>
using namespace std;

// Complexity: O(N + N * logN + 50 * 50 * K)

const int MAXN = 524288 * 2;
const int MAXM = 524288 * 2;
const int MAXK = 65536;

struct edge
{
    int to;
    int cost;
    
    edge ()
    {
    }
    
    edge (int to, int cost)
    {
        this->to = to;
        this->cost = cost;
    }
};

int anc [64];
int want [64];
int idx [MAXN / 2];
int stack [MAXN];
int path [MAXN / 2];
int prenum [MAXN / 2];
int parent [MAXN / 2], p3 = 0;
int order [MAXN], p4 = 0;
//bool mark [MAXN / 2];
int depth [MAXN / 2];
int least [20 + 1][MAXN];
vector <edge> g [MAXN / 2];

int N, M, K;

inline int rmq_query (int i, int j)
{
    i = prenum [i];
    j = prenum [j];
    if (i > j)
        swap (i, j);
    
    int len = j - i + 1;
    int bits = 31 - __builtin_clz (len);
    if (depth [least [bits][i]] < depth [least [bits][j + 1 - (1 << bits)]])
        return least [bits][i];
    else
        return least [bits][j + 1 - (1 << bits)];
        
//    printf ("%d %d\n", depth [least [1
//    printf ("[%d, %d] && [%d, %d]\n", i, bits, j + 1 - (1 << bits), bits);
//    printf ("depth 1 = %d\n", depth [least [i][bits]]);
//    printf ("depth 2 = %d\n", depth [least [j + 1 - (1 << bits)][bits]]);
//    exit (0);
/*    int len = j - i + 1;
    int bits = 31 - __builtin_clz (len);
    if (depth [least [bits][i]] < depth [least [bits][j + 1 - (1 << bits)]])
        return depth [least [bits][i]];
    else
        return depth [least [bits][j + 1 - (1 << bits)]];*/
}
void precompute ()
{
    // Traverse the tree and get all information which we will need afterwards:
    stack [p3 ++] = 0;
    order [p4 ++] = 0;
    depth [0] = 0;
    path [0] = 0;
    prenum [0] = 0;
    int at;
    while (p3 > 0)
    {
        at = stack [p3 - 1];
        if (idx [at] >= g [at].size ())
        {
            order [p4 ++] = parent [at];
            p3 --;
        }
        else
        {
            prenum [g [at][idx [at]].to] = p4;
            depth [g [at][idx [at]].to] = depth [at] + 1;
            path [g [at][idx [at]].to] = path [at] + g [at][idx [at]].cost;
            stack [p3 ++] = g [at][idx [at]].to; idx [at] ++;
            order [p4 ++] = stack [p3 - 1];
        }
    }
/*    for (int i = 0; i < p4; ++i)
        printf ("%d ", order [i] + 1);
    puts ("");
    for (int i = 0; i < p4; ++i)
        printf ("%d ", depth [order [i]]);
    puts ("");
    for (int i = 0; i < N; ++i)
    {
        printf ("depth [%d] = %d\n", i + 1, depth [i]);
        printf ("path [%d] = %d\n", i + 1, path [i]);
        printf ("prenum [%d] = %d\n", i + 1, prenum [i]);
    }*/
    
    // RMQ: O(N * logN)
    for (int i = 0; i < p4; ++i)
        least [0][i] = order [i];
    for (int i = 1; i <= 20; ++i)
        for (int j = 0; j + (1 << i) <= p4; ++j)
        {
            if (depth [least [i - 1][j]] < depth [least [i - 1][j + (1 << (i - 1))]])
                least [i][j] = least [i - 1][j];
            else
                least [i][j] = least [i - 1][j + (1 << (i - 1))];
            
//            printf ("least [%d, %d] = %d\n", j, j + (1 << i) - 1, depth [least [i][j]]);
        }
    /*
    while (true)
    {
        // Get LCA-node between nodes a and b.
        int a, b, t;
        scanf ("%d %d", &a, &b);
        a = prenum [a - 1];
        b = prenum [b - 1];
        if (a > b) { t = a; a = b; b = t; }
        int len = b - a + 1;
        int bits = 31 - __builtin_clz (len);
        if (depth [least [a][bits]] < depth [least [a + len - (1 << bits)][bits]])
            printf ("%d\n", least [a][bits]);
        else
            printf ("%d\n", least [a + len - (1 << bits)][bits]);
//        printf ("%d %d\n", least [a][bits], least [b - (1 << bits)][bits]);
    }*/
    /*
    for (int i = 0; i < p4; ++i)
        for (int j = i + 1; j < p4; ++j)
        {
            printf ("min [%d, %d] = %d\n", i, j, rmq_query (i, j));
            int got = rmq_query (i, j);
            int res = 10000;
            for (int k = i; k <= j; ++k)
                res <?= depth [order [k]];
            assert (got == res);
        }
        */
/*
    for (int i = 0; i < N; ++i)
        for (int j = 0; j < N; ++j) // j = 0
        {
            printf ("[%d, %d] -> %d\n", i + 1, j + 1, rmq_query (i, j) + 1);
        }*/

    int cnt;
    int curDepth;
    int ancestor;
    int curAncestor;
    long long res;
    for (int i = 0; i < K; ++i)
    {
        scanf ("%d", &cnt);
        for (int j = 0; j < cnt; ++j)
            scanf ("%d", &want [j]);
        /*    
        res = 0;
        for (int a = 0; a < cnt; ++a)
        {
            ancestor = 0;
            curDepth = 0;
            for (int b = 0; b < cnt; ++b)
            {
                if (a == b)
                   continue;
                curAncestor = rmq_query (want [a] - 1, want [b] - 1);
//                printf ("anc [%d, %d] = %d\n", want [a], want [b], curAncestor + 1);
                if (depth [curAncestor] > curDepth)
                {
                    ancestor = curAncestor;
                    curDepth = depth [curAncestor];
                }
            }
            res += path [ancestor] / 2 + (path [want [a] - 1] - path [ancestor]);
//            printf ("1 -> %d -> %d\n", ancestor + 1, want [a]);
        }
        printf ("%lld\n", res);
//        cout << res << "\n";
        */
        for (int j = 0; j < cnt; ++j)
            anc [j] = 0;
        for (int a = 0; a < cnt; ++a)
            for (int b = a + 1; b < cnt; ++b)
            {
                curAncestor = rmq_query (want [a] - 1, want [b] - 1);
                if (depth [anc [a]] < depth [curAncestor] || depth [anc [b]] < depth [curAncestor])
                {
                    anc [a] = anc [b] = curAncestor;
                }
            }
        res = 0;
        for (int j = 0; j < cnt; ++j)
            res += path [anc [j]] / 2 + path [want [j] - 1] - path [anc [j]];
        printf ("%lld\n", res);
    }
}

int main ()
{
    memset (parent, -1, sizeof (parent));
    parent [0] = 0;
    scanf ("%d %d %d", &N, &M, &K);
    for (int i = 0; i < M; ++i)
    {
        int u, v, w;
        scanf ("%d %d %d", &u, &v, &w);
        parent [v - 1] = u - 1;
        g [u - 1].push_back (edge (v - 1, w));
    }
    // Haven't read the whole input. -> The queries will be answered one by one!
//    for (int i = 0; i < N; ++i)
//        for (int j = 0; j < g [i].size (); ++j)
//            printf ("%d %d\n", i + 1, g [i][j].to + 1);
//    for (int i = 0; i < N; ++i)
//        printf ("%d %d\n", i + 1, parent [i] + 1);
    precompute ();
    return 0;
}
